/**
 * Windows Hashing/Checksumming Library
 * Last modified: 2009/03/04
 * Copyright (C) Kai Liu.  All rights reserved.
 *
 * This is a wrapper for the MD4, MD5, SHA1, and CRC32 algorithms supported
 * natively by the Windows API.
 *
 * WinHash.c is needed only if the WH*To* or WH*Ex functions are used.
 **/

#include "WinHash.h"


/*******************************************************************************
 *
 *
 * WH*To* hex string conversion functions
 *
 *
 ******************************************************************************/


BOOL WHAPI WHHexToByte( PCTSTR pszSrc, PBYTE pbDest, UINT cchHex )
{
	while (cchHex)
	{
		BYTE nybble = LOBYTE(*pszSrc);

		// If this is UTF-16, the high byte must always be zero
		if (sizeof(TCHAR) == 2 && HIBYTE(*pszSrc))
			return(FALSE);

		// Now convert the hex character into a 4-bit nybble
		if (nybble < '0')
		{
			return(FALSE);
		}
		else if (nybble > TEXT('9'))
		{
			// Ensure that this is lower-case
			nybble |= 0x20;

			// The x86-64 compiler will allocate registers inefficiently if
			// these two comparisons are joined with an logical OR
			if (nybble < 'a') return(FALSE);
			if (nybble > 'f') return(FALSE);

			nybble -= 'a' - 10;
		}
		else
		{
			nybble -= '0';
		}

		// Save to the resulting byte array
		if (!(cchHex & 1))
		{
			*pbDest = nybble << 4;
		}
		else
		{
			*pbDest |= nybble;
			++pbDest;
		}

		++pszSrc;
		--cchHex;
	}

	return(TRUE);
}

PTSTR WHAPI WHByteToHex( PCBYTE pbSrc, PTSTR pszDest, UINT cchHex, UINT8 uCaseMode )
{
	while (cchHex)
	{
		BYTE nybble = *pbSrc;

		// Isolate the appropriate 4-bit nybble
		if (!(cchHex & 1))
		{
			nybble >>= 4;
		}
		else
		{
			nybble &= 0x0F;
			++pbSrc;
		}

		// Translate to a character in the range of '0'-'?'
		nybble |= 0x30;

		// Translate from the ':'-'?' range to the 'A'-'F' or 'a'-'f' ranges
		if (nybble > '9')
		{
			nybble += 'A' - ('0' + 10);
			nybble |= uCaseMode;
		}

		*pszDest = nybble;

		++pszDest;
		--cchHex;
	}

	*pszDest = 0;

	return(pszDest);
}


/*******************************************************************************
 *
 *
 * WH*Ex functions
 *
 *
 ******************************************************************************/


VOID WHAPI WHInitEx( PWHCTXEX pContext )
{
	if (pContext->fAlgs & WHEX_CHECKCRC32)
		WHInitCRC32(&pContext->ctxCRC32);

	if (pContext->fAlgs & WHEX_CHECKMD4)
		WHInitMD4(&pContext->ctxMD4);

	if (pContext->fAlgs & WHEX_CHECKMD5)
		WHInitMD5(&pContext->ctxMD5);

	if (pContext->fAlgs & WHEX_CHECKSHA1)
		WHInitSHA1(&pContext->ctxSHA1);

	if (pContext->fAlgs & WHEX_CHECKED2K)
		WHInitED2K(&pContext->ctxED2K);
}

VOID WHAPI WHUpdateEx( PWHCTXEX pContext, PCBYTE pbIn, UINT cbIn )
{
	if (pContext->fAlgs & WHEX_CHECKCRC32)
		WHUpdateCRC32(&pContext->ctxCRC32, pbIn, cbIn);

	if (pContext->fAlgs & WHEX_CHECKMD4)
		WHUpdateMD4(&pContext->ctxMD4, pbIn, cbIn);

	if (pContext->fAlgs & WHEX_CHECKMD5)
		WHUpdateMD5(&pContext->ctxMD5, pbIn, cbIn);

	if (pContext->fAlgs & WHEX_CHECKSHA1)
		WHUpdateSHA1(&pContext->ctxSHA1, pbIn, cbIn);

	if (pContext->fAlgs & WHEX_CHECKED2K)
		WHUpdateED2K(&pContext->ctxED2K, pbIn, cbIn);
}

VOID WHAPI WHFinishEx( PWHCTXEX pContext, PWHRESULTEX pResults )
{
	if (pResults == NULL)
		pResults = &pContext->results;

	if (pContext->fAlgs & WHEX_CHECKCRC32)
		WHFinishCRC32Ex(&pContext->ctxCRC32, pResults->CRC32);

	if (pContext->fAlgs & WHEX_CHECKMD4)
		WHFinishMD4Ex(&pContext->ctxMD4, pResults->MD4);

	if (pContext->fAlgs & WHEX_CHECKMD5)
		WHFinishMD5Ex(&pContext->ctxMD5, pResults->MD5);

	if (pContext->fAlgs & WHEX_CHECKSHA1)
		WHFinishSHA1Ex(&pContext->ctxSHA1, pResults->SHA1);

	if (pContext->fAlgs & WHEX_CHECKED2K)
		WHFinishED2KEx(&pContext->ctxED2K, pResults->ED2K);
}
